﻿/* =========================================================================
NAME: Footnoter 0.9

AUTHOR: goldyz@yandex.ru
www.4may.by.ru

DATE  : 16.07.2004

COMMENT:
Скрипт Footnoter 0.9 автоматизирует некоторые из ручных операций выполняемых верстальщиком в программах  верстки не обладающих штатными средствами работы со сносками.

Верстальщик
при оформлении сносок в среде InDesign, Quark, PageMaker:
1. копирует текст сноски из внешнего источника или вырезает из уже заверстанного в публикацию материала;
2. создает новый фрейм и вставляет туда содержимое буфера обмена;
3. ставит сносочный фрейм вниз полосы набора и подтягивает остальные фреймы "туда-сюда";
4. расставляет индексы сносок в абзацах сносочного фрейма и повторяет их же в основном тексте.

Footnoter 0.9:
1. Вырезает предварительно выделенный в верстке текст (myDoc.selection=text).
2. Создает фрейм (myFootFrame) для сносок внизу полосы набора, вставляет туда содержимое буфера обмена, не обращая при этом внимания на источник содержимого, и присваивает созданному фрейму значение обтекания текстом ini_TopWrapOffset (Footnoter.ini). Размеры создаваемого фрейма определяется согласно следующим правилам:
	- выделен текстовый фрейм (myDoc.selection=textFrame), текст (myDoc.selection=text) или курсор в тексте (myDoc.selection=insertionPoint) - фрейм со сносками ставится по ширине исходного текстового фоейма внизу полосы набора
	- ничего не выделено - сносочный фрейм ставится по ширине полосы набора, либо в крайнюю правую/крайнюю левую колонку полосы (в случае многоколонного макета и включенной в файле Footnoter.ini опции ini_FootByColumn = true);
	- высота фрейма подгоняется под добавляемый текст только в случае одиночности сносочного фрейма в цепочке фреймов (myFootStory).
3. Добавляет в публикацию абзацные стили для сносок Footnoter1, Footnoter2 и символьные FootnoterC1, FootnoterC2 с дальнейшей проверкой их наличия (однажды добавив, удовлетворяется, увидев имя - стили в дальнейшем можно редактировать как угодно);
4. При обнаружении на полосе фрейма со сносками, созданного при предыдущих запусках скрипта, добавляет содержимое буфера обмена в готовый фрейм с автонумерацией вновь добавляемого текста. Наличие фрейма определяется через свойство label текстового сносочного фрейма: myFootFrame.label = "Footnote" (присваивается каждому вновь создающемуся сносочному фрейму).
5. Расставляет индексы сносок с возможностью перенумерации и изменения типа индексов. Если верстальщик изменяет индекс (номер) первого абзаца сноски на развороте и/или значение ini_FootSymbolsType в файле Footnoter.ini, то при следующем запуске скрипта остальные индексы переопределятся автоматически.
6. Добавляет индекс сноски в основной текст (только в случае myDoc.selection=text или myDoc.selection=insertionPoint).

Footnoter 0.9 НЕ УМЕЕТ:
1. Перенумеровывать и вообще осуществлять какой-либо контроль за индексами в основном материале.
2. Расставлять индексы сносок вида "i,ii,iii,iv..." (латинская нумерация)
3. Разверстывать сносочные фреймы по нескольким разворотам и привязывать их к основному материалу.

Очевидно, что эти возможности должны быть реализованы в ближайшем будущем. Не менее очевидно, что свойствами JavaScript это сделать обременительно. Нужен как минимум класс объектов - Footnote со всем множеством свойств и методов (а их действительно множество). Возможно, что реализация всего перечисленного является вопросом скорее для плагино- нежели скриптописателей. В идеале в InDesign должен появиться инструментарий работы со сносками, аналогичный MS Word. Не очень понятно, почему все это не было реализовано в ID CS.
Настоящий скрипт выходит под номером 0.9, может содержать ошибки и не является каким-либо конечным продуктом. Скорее это пища для размышлений dtp-community.


============================================================================ */

$.level=0; //бесплатная версия (freeware то бишь;-)

// =============================
// ОСНОВНОЙ МОДУЛЬ
// MAIN MODULE
// =============================
with (app) {
	
	//Подключаем ini-файл и языковой модуль
	// ini-file and language module including
	var myIniFile = new File (activeScript.path + "/Footnoter.ini");
	if (!myIniFile.exists) {
		alert("Can\'t find ini file!");
		exit (); }
	else {
		eval ("//@include \'Footnoter.ini\';"); }
	var myLangFile = new File (activeScript.path + "/Footnoter.slang");
	if (!myLangFile.exists) {
		alert("Can\'t find lang file!");
		exit (); }
	else {
		eval ("//@include \'Footnoter.slang\';"); }

	//Проверка наличия открытого документа
	//Open document control
	if (app.documents.length < 1) {
		var myDoc = app.documents.add();}
	
	var myDoc = app.activeDocument;
	var myOldOrigin = myDoc.viewPreferences.rulerOrigin;
	var myOldHorUnits = myDoc.viewPreferences.horizontalMeasurementUnits;
	var myOldVerUnits = myDoc.viewPreferences.verticalMeasurementUnits;
	var myOldZeroPoint = myDoc.zeroPoint;
	var myPage = myDoc.layoutWindows.item(0).activePage;
	var myPH = myDoc.documentPreferences.pageHeight;
	var myPW = myDoc.documentPreferences.pageWidth;
	var myMargins = myPage.marginPreferences;
	var myColumns = myMargins.columnCount;
	var myGutter = myMargins.columnGutter;
	var myTempBounds = Array(myMargins.top, myMargins.left, myPH - myMargins.bottom, myPW - myMargins.right); // координаты полосы набора
	var myFootFlag = false;
	var myFootFrameID = 0;
	var myInsertionPointIndex = -1;
	var FootSymbolList = ini_FootSymbols.split(","); // *,†,‡,§ symbols array
	var upLatin = ini_upLatin.split(","); // A,B,C...Z array
	var smLatin = ini_upLatin.split(","); // a,b,c...z array
	
	myDoc.viewPreferences.rulerOrigin = 1380143215; //pageOrigin
	myDoc.viewPreferences.horizontalMeasurementUnits = 2053991795; //mm
	myDoc.viewPreferences.verticalMeasurementUnits = 2053991795; //mm
	myDoc.zeroPoint = Array(0,0);
	
	//рассчитываем размер сносочного фрейма для многоколонной верстки
	//calculating the Footframe size fot multicolumn layout
	if ((ini_FootByColumn == true) & (myColumns > 1)){
		myColumnWidth = ((myPW - myMargins.left - myMargins.right)/myColumns) - (myGutter - myGutter/myColumns);
		switch (myPage.index) 
		{  case 0:
		      myTempBounds[3] = myTempBounds[1]+myColumnWidth; 
		   case 1:
		      myTempBounds[1] = myTempBounds[3]-myColumnWidth;
		   default:
		      myTempBounds[3] = myTempBounds[1]+myColumnWidth;  }
	}

	
	//обрабатываем выбор пользователя
	//working with user selection
	if (myDoc.selection.length > 0) {
		var mySelection = myDoc.selection[0];
		switch (mySelection.constructor.name) {
			case "InsertionPoint":
				var myTextFrame = mySelection.parentTextFrame;
				var myPage = myTextFrame.parent;
				var myTempBounds = myTextFrame.geometricBounds;
				var myFootFrameID = isFootFrame(myPage);
				var myInsertionPointIndex = mySelection.index;
				break;
			case "Text":
				var myTextFrame = mySelection.parentTextFrame;
				var myPage = myTextFrame.parent;
				var myFootFrameID = isFootFrame(myPage);
				var myTempBounds = myTextFrame.geometricBounds;
				cut();
				var myInsertionPointIndex = myDoc.selection[0].index;
				break;				
			case "TextFrame":
				var myTextFrame = mySelection;
				var myPage = myTextFrame.parent;
				var myFootFrameID = isFootFrame(myPage);
				var myTempBounds = myTextFrame.geometricBounds;			
				break;
			}
		}
		else {
			var myFootFrameID = isFootFrame(myPage);}
		

	
	
	//aнализ наличия стилей Footnote в публикации
	//analysis of Footnote styles in publication
	var myFootParaStyle1 = isFootStyle(myDoc.paragraphStyles, ini_Footnote1);
	var myFootParaStyle2 = isFootStyle(myDoc.paragraphStyles, ini_Footnote2);
	var myFootCharStyle1 = isFootStyle(myDoc.characterStyles, ini_FootnoteC1);
	var myFootCharStyle2 = isFootStyle(myDoc.characterStyles, ini_FootnoteC2);
	

	// если сносок не обнаружено, добавляем сносочный фрейм
	// footnote-frame adding if it absent
	if (myFootFrameID == 0) {
		var myFootFrame = myPage.textFrames.add();
		var myFootStory = myFootFrame.parentStory;
		myFootFrame.label = "Footnote";
		myFootFrame.textFramePreferences.textColumnCount = 1;
		myFootFrame.geometricBounds = myTempBounds;
	}
	// если сноска на полосе уже есть
	// if footnote-frame already exist 
	else {
		var myFootFrame = myPage.textFrames.itemByID(myFootFrameID);
		var myFootStory = myFootFrame.parentStory;
		var myTempBounds = myFootFrame.geometricBounds;
		if (myFootStory.textFrames.length == 1) {
			myFootFrame.textWrapPreferences.textWrapType=1852796517; // noWrap
			myFootFrame.geometricBounds = Array(myMargins.top, myTempBounds[1], myPH-myMargins.bottom, myTempBounds[3]);
		}
	myFootStory.insertionPoints.lastItem().contents = "\u000D";
	}
	
	
	
	//определяем тип индекса и последнее значение индекса в сноске
	//define of first footnote-index value
	if ((myFootFrameID == 0) && ini_FootnoteNumbering) {
		myFootFrame.insertionPoints[0].contents = getFootSymbols(1); 
		var myFootStoryIndex = Array(1,1,ini_FootSymbolsType);
		}
	else {
		var myFootStoryIndex = (myFootStory.label).split(",");
		var myDelta = parseInt(myFootStoryIndex[1]) - parseInt(myFootStoryIndex[0]);
		myFootStoryIndex[0] = getFootNumberString(myFootStory.paragraphs.item(0), false);
		myFootStoryIndex[1]= myFootStoryIndex[0] + myDelta + 1;
		myFootStoryIndex[2] = parseInt(myFootStoryIndex[2]);
		myFootStory.insertionPoints.lastItem().contents = getFootSymbols(myFootStoryIndex[1]);
		}
	
	
	//добавляем содержимое буфера в сноску с проверкой на корректность
	//pasting clipboard-content to end of footnote-frame
	myDoc.selection = myFootStory.insertionPoints[myFootStory.insertionPoints.length-1];
	try {
		paste();}
	catch(err) {
		alert(lang_BadClipboardContent);
		if (myFootFrameID == 0) {
			myFootFrame.remove(); }
	}
	
	
	//=============================
	//обрабатываем абзацы сноски
	//work with footnotes paragraphs
	//=============================
	//удаляем мусор (парные абзацы и пробелы, пробелы в начале и в конце абзаца)
	//deleting the double para-symbols and spaces, start and end para-space
	myFindWhatArray = ini_FindWhat.split(",");
	myFindToArray = ini_FindTo.split(",");
	for (i = 0; i < ini_FindWhat.length; i++) {
		myFoundItems = myFootStory.search(myFindWhatArray[i], false, false, myFindToArray[i]);
		}
	if (myFootStory.characters.lastItem().contents == "\p") {
		myFootStory.characters.lastItem().remove();}
	
	//присваиваем всем абзацам сноски стиль Footnote2 и расставляем номера сносок
	//applying para-style Footnote2 to all paragraphs and seting footnote-index
	myFootStory.appliedParagraphStyle = myFootParaStyle2.name;
	myDelta = 0;
	for (myCounter = 0; myCounter < myFootStory.paragraphs.length; myCounter++) {
		var myPara =  myFootStory.paragraphs.item(myCounter);
		if (getFootNumberString(myPara, true) > 0) {
			myPara.insertionPoints.firstItem().appliedCharacterStyle = ini_FootnoteC1;
			myPara.insertionPoints.firstItem().contents = getFootSymbols(myFootStoryIndex[0]+myDelta);
			myDelta++;
		}
	}
	myFootStoryIndex[1] = myFootStoryIndex[0]+myDelta;
	myFootStoryIndex[2] = ini_FootSymbolsType;
	myFootStory.label=myFootStoryIndex[0]+"" + "," + myFootStoryIndex[1] + "" + "," + myFootStoryIndex[2]+"";
	
	//присваиваем первому абзацу сноски стиль Footnote1
	//applying para-style Footnote1 to first paragraph
	for (i=0; i <= myFootStory.textFrames.length; i++) {
		myFootStory.textFrames.firstItem().paragraphs.item(0).appliedParagraphStyle=ini_Footnote1;
	}
		
	
	// ужимаем сносочный фрейм до текста и ставим его вниз полосы набора
	// fitting footnote-frame and move it to bottom of type area 
	if (myFootStory.textFrames.length == 1) {
		var myFW = myFootFrame.geometricBounds[3]-myFootFrame.geometricBounds[1];
		myFootFrame.fit(1718906723);
		var myFH = myFootFrame.geometricBounds[2]-myFootFrame.geometricBounds[0];
		var myBottomY = myPH - myMargins.bottom;
		var myTopY = myBottomY - myFH;
		var myLeftX = myTempBounds[3]-myFW;
		var myRightX = myTempBounds[3];
		myFootFrame.geometricBounds = Array(myTopY, myLeftX, myBottomY, myRightX);
		myFootFrame.textWrapPreferences.textWrapType=1651729523; //boundingBoxTextWrap
		myFootFrame.textWrapPreferences.textWrapOffset = Array(ini_TopWrapOffset, 0, 0, 0);
	}
	myFootFrame.select(SelectionOptions.replace);
	myDoc.layoutWindows.item(0).zoomPercentage = 100;
	myDoc.selection = null;
	

	//ставим номер сноски в основной текст
	//adding footnote-index to main text
	if (myInsertionPointIndex > -1) {
		myTextFrame.insertionPoints.item(myInsertionPointIndex).appliedCharacterStyle = myFootCharStyle1;
		var myBodyFootSymbols = getFootSymbols(myFootStoryIndex[1]);
		myBodyFootSymbols = myBodyFootSymbols.substring(0, myBodyFootSymbols.length - 1);
		myTextFrame.insertionPoints.item(myInsertionPointIndex).contents = myBodyFootSymbols;
	}
		
	//возвращение исходных параметров
	//return of the initial operation factors 
	myDoc.viewPreferences.rulerOrigin = myOldOrigin;
	myDoc.viewPreferences.horizontalMeasurementUnits = myOldHorUnits;
	myDoc.viewPreferences.verticalMeasurementUnits = myOldVerUnits;
	myDoc.zeroPoint = myOldZeroPoint;
	exit ();
}



// =======================================
// Функция получения строки-индекса сноски
// Function getFootNumberString
// =======================================
function getFootNumberString (Para, editFlag) {
	var myString = "";
	var myNumber = 0;
	var myChar = Para.characters.item(0).contents;
	var myFound = Para.search(ini_FootIndentSymbol, false, false, ini_FootIndentSymbol);
	if (myFound.length > 0) {
		var myDelPosition = myFound[0].index - Para.insertionPoints.item(0).index;
		myString = (Para.contents).substring(0,myDelPosition);
		if (editFlag == true) {
			var myCounter = myDelPosition;
			while (myCounter >=0) {
				Para.characters.item(myCounter).contents = "";
				myCounter--; }
		}
		try {
			myNumber = parseInt(myString);
			if (myNumber >= 0) {
				myChar =""; }
			else { myNumber = myString.length; }
		}
		catch (err) { myNumber = 0; };
	}
	//alert (myNumber);
	return myNumber;
}


// =============================================
// Функция формирования строки-индекса сноски
// Function of footnote-index string composition
// =============================================
function getFootSymbols(myNum) {
	var myFootSymbols = "";
	switch (ini_FootSymbolsType) {
		case 1:
			myFootSymbols = myNum +"";
			break;
		case 2:
			var myRound = Math.floor((myNum-1)/26);
			myFootSymbols = getString(myRound+1, upLatin[myNum-myRound*26-1]);
			break;
		case 3:
			var myRound = Math.floor((myNum-1)/26);
			myFootSymbols = getString(myRound+1, smLatin[myNum-myRound*26-1]);
			break;
	 	case 6:
			myFootSymbols = getString(myNum, FootSymbolList[0]);
			break;
	    case 7:
	    	var myRound = Math.floor((myNum-1)/4);
			myFootSymbols = getString(myRound+1, FootSymbolList[myNum-myRound*4-1]);
	    	break;
		default:
			FootSymbols = myNum +"";
	}
return (myFootSymbols+ini_FootIndentSymbol);
}

function getString(NumberOfSymbol, Ch){
	var myString = "";
	for (i=1; i<= NumberOfSymbol; i++) {
	      myString = myString + Ch; }
return myString;
}

// ==============================================
// Функция определения наличия фрейма со сносками
// Function of footnote-frame finding on page
// ==============================================
function isFootFrame (Page) {
	var TextFrameID = 0;
	for (myCounter = 0; myCounter < Page.pageItems.length; myCounter++) {
		if (Page.pageItems[myCounter].label == "Footnote") {
			if (confirm(lang_FootnoteIsOnPage) == true) {
				var TextFrameID = Page.pageItems[myCounter].getElements()[0].id;
			}
			else exit();
		}
	}
return TextFrameID;
}

// ==================================================
// Функция добавления сносочных стилей
// Function of Footnote-parastyles finding and adding
// ==================================================
function isFootStyle (stylesRange, myFootStyleName) {
	var myStyle = stylesRange.item(myFootStyleName);
	try { myStyle.name; }
	catch (err){
		myStyle = stylesRange.add({name:myFootStyleName});
	  with (myStyle) {
		switch (myFootStyleName) {
		   case ini_FootnoteC1:
		   		position = 1936749411; //superscript
			  	break;
		   case ini_FootnoteC2:
		      	position = 1936749411; //superscript
		      	break;
		   case ini_Footnote1:
		   		pointSize = ini_pointSize;
				leading = ini_leading;
				alignToBaseline = ini_alignToBaseline;
				position = ini_position;
				appliedFont = ini_appliedFont;
				fontStyle = ini_fontStyle;
				justification = ini_justification;
				hyphenation = ini_hyphenation;
				leftIndent = ini_leftIndent;
				firstLineIndent = ini_firstLineIndent;
				rightIndent = ini_rightIndent;
				ruleAbove = ini_ruleAbove;
				ruleAboveColor = ini_ruleAboveColor;
				ruleAboveLeftIndent = ini_ruleAboveLeftIndent;
				ruleAboveLineWeight = ini_ruleAboveLineWeight;
				ruleAboveOffset = ini_ruleAboveOffset;
				spaceAfter = ini_spaceAfter;
				if (ini_FootnoteNumbering == false){
					firstLineIndent = 0;
					leftIndent = 0; }
				break;
		   case ini_Footnote2:
		   		basedOn = stylesRange.item(ini_Footnote1);
				ruleAbove = false;
				break;
			}
		}
	}
return myStyle;
}